from .. import ImageNormalizer


def create_random_img_array(img_height, img_width):
    import numpy as np
    return np.random.randint(0, 256, (img_height, img_width, 3))


def test_transform_results():
    img_size = 217
    target_size = 224
    crafter = ImageNormalizer(target_size=target_size)
    img_array = create_random_img_array(img_size, img_size)
    crafted_doc = crafter.craft(img_array)
    assert crafted_doc["blob"].shape == (224, 224, 3)



import numpy as np

from .. import AudioSlicer


def test_segment_mono():
    n_frames = 100
    frame_length = 2048
    signal_orig = np.random.randn(frame_length * n_frames)

    crafter = AudioSlicer(frame_length)
    crafted_chunks = crafter.craft(signal_orig, 0)

    assert len(crafted_chunks) == n_frames


def test_segment_stereo():
    n_frames = 100
    frame_length = 2048
    signal_orig = np.random.randn(2, frame_length * n_frames)

    crafter = AudioSlicer(frame_length)
    crafted_chunks = crafter.craft(signal_orig, 0)

    assert len(crafted_chunks) == n_frames * 2



from .. import JiebaSegmenter


def test_jieba_crafter():
    jieba_crafter = JiebaSegmenter(mode='accurate')
    text = '今天是个大晴天！安迪回来以后，我们准备去动物园。'
    crafted_chunk_list = jieba_crafter.craft(text, 0)
    assert len(crafted_chunk_list) == 14



import numpy as np

from .. import AudioNormalizer


def test_audionormalizer():
    """here is my test code

    https://docs.pytest.org/en/stable/getting-started.html#create-your-first-test
    """
    signal_orig = np.random.randn(2, 31337)

    crafter = AudioNormalizer()
    crafted_doc = crafter.craft(signal_orig, 0)

    signal_norm = crafted_doc["blob"]
    assert signal_norm.shape == signal_orig.shape
    assert np.min(signal_norm) == -1
    assert np.max(signal_norm) == 1



import numpy as np

from .. import SlidingWindowAudioSlicer


def test_sliding_window_mono():
    n_frames = 100
    frame_length = 2048
    signal_orig = np.random.randn(frame_length * n_frames)

    crafter = SlidingWindowAudioSlicer(frame_length, frame_length // 2)
    crafted_chunks = crafter.craft(signal_orig, 0)

    assert len(crafted_chunks) == n_frames * 2 - 1


def test_sliding_window_stereo():
    n_frames = 100
    frame_length = 2048
    signal_orig = np.random.randn(2, frame_length * n_frames)

    crafter = SlidingWindowAudioSlicer(frame_length, frame_length // 2)
    crafted_chunks = crafter.craft(signal_orig, 0)

    assert len(crafted_chunks) == (n_frames * 2 - 1) * 2



from .. import AudioReader


def test_1():
    import librosa
    audio_file_path = librosa.util.example_audio_file()

    crafter = AudioReader()
    crafted_doc = crafter.craft(audio_file_path, 0)

    signal = crafted_doc['blob']
    assert signal.shape == (2, 1355168)



import os
import shutil
import mock
import numpy as np
import pytest

from .. import VideoTorchEncoder
from jina.executors.metas import get_default_metas
from jina.executors import BaseExecutor
from typing import TypeVar

batch_size = 2
input_dim = 224
output_dim = 512
channel = 3
num_frames = 10


def rm_files(file_paths):
    for file_path in file_paths:
        if os.path.exists(file_path):
            if os.path.isfile(file_path):
                os.remove(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path, ignore_errors=False, onerror=None)


class MockModels:
    def r3d_18(pretrained=True, *args, **kwargs):
        return MockModel()


class MockModel:
    def eval(self):
        return self

    def to(self, *args, **kwargs):
        pass

class MockFeature:
    def numpy(self):
        np.random.seed(0)
        return np.random.random((batch_size, output_dim))

    def detach(self):
        return self

    def cpu(self):
        return self


def get_encoder(*args, **kwargs):
    metas = get_default_metas()
    if 'JINA_TEST_GPU' in os.environ:
        metas['on_gpu'] = True
    return VideoTorchEncoder(metas=metas)

def _test_encoding_results(*args, **kwargs):
    encoder = get_encoder(*args, **kwargs)
    test_data = np.random.rand(batch_size, num_frames, channel, input_dim, input_dim)
    encoded_data = encoder.encode(test_data)
    assert encoded_data.shape == (batch_size, output_dim)
    rm_files([encoder.save_abspath, encoder.config_abspath])

@mock.patch('torchvision.models.video', return_value=MockModels())
@mock.patch.object(VideoTorchEncoder, '_get_features', return_value=MockFeature())
def test_encoding_results(*args, **kwargs):
    _test_encoding_results(*args, **kwargs)

@mock.patch('torchvision.models.video', return_value=MockModels())
@mock.patch.object(VideoTorchEncoder, '_get_features', return_value=MockFeature())
def test_save_and_load(*args, **kwargs):
    encoder = get_encoder()
    test_data = np.random.rand(batch_size, num_frames, channel, input_dim, input_dim)
    encoded_data_control = encoder.encode(test_data)
    encoder.touch()
    encoder.save()
    assert os.path.exists(encoder.save_abspath)
    encoder_loaded = BaseExecutor.load(encoder.save_abspath)
    encoded_data_test = encoder_loaded.encode(test_data)
    assert encoder_loaded.channel_axis == encoder.channel_axis
    np.testing.assert_array_equal(encoded_data_control, encoded_data_test)
    rm_files([encoder.save_abspath, encoder.config_abspath])

@mock.patch('torchvision.models.video', return_value=MockModels())
@mock.patch.object(VideoTorchEncoder, '_get_features', return_value=MockFeature())
def test_save_and_load_config(*args, **kwargs):
    encoder = get_encoder()
    encoder.save_config()
    assert os.path.exists(encoder.config_abspath)
    encoder_loaded = BaseExecutor.load_config(encoder.config_abspath)
    assert encoder_loaded.channel_axis == encoder.channel_axis
    rm_files([encoder.save_abspath, encoder.config_abspath])

def test_pool_fn():
    test_data = np.random.rand(batch_size, num_frames, channel, input_dim, input_dim)
    encoder = get_encoder()
    encoded_data = encoder.pool_fn(test_data, axis=(2, 3))
    assert encoded_data.ndim == test_data.ndim - 2

@pytest.mark.skipif('JINA_TEST_PRETRAINED' not in os.environ, reason='skip the pretrained test if not set')
def test_video_torch_encode_with_pretrained_model():
    _test_encoding_results()


import numpy as np

from .. import ArrayStringReader


def test_arraystringreader():
    """here is my test code

    https://docs.pytest.org/en/stable/getting-started.html#create-your-first-test
    """
    size = 8
    sample_array = np.random.rand(size).astype('float32')
    text = ','.join([str(x) for x in sample_array])

    reader = ArrayStringReader()
    crafted_doc = reader.craft(text, 0)

    assert crafted_doc['blob'].shape[0] == size
    np.testing.assert_array_equal(crafted_doc['blob'], sample_array)



import os
import shutil

import numpy as np
import pytest
from jina.executors import BaseExecutor
from jina.executors.metas import get_default_metas

from .. import LaserEncoder


def rm_files(tmp_files):
    for k in tmp_files:
        if os.path.exists(k):
            if os.path.isfile(k):
                os.remove(k)
            elif os.path.isdir(k):
                shutil.rmtree(k, ignore_errors=False, onerror=None)


os.environ["TEST_WORKDIR"] = os.getcwd()

metas = get_default_metas()
if "JINA_TEST_GPU" in os.environ:
    metas["on_gpu"] = True

encoders = [
    LaserEncoder(language="en", metas=metas),
    LaserEncoder(language="zh", metas=metas),
    LaserEncoder(language="ja", metas=metas),
]


@pytest.mark.parametrize("encoder", encoders)
def test_encoding_results(encoder):
    target_output_dim = 1024
    test_data = np.array(["it is a good day!", "the dog sits on the floor."])
    encoded_data = encoder.encode(test_data)
    assert encoded_data.shape == (2, target_output_dim)


@pytest.mark.parametrize("encoder", encoders)
def test_save_and_load(encoder):
    encoder.save_config()
    assert os.path.exists(encoder.config_abspath)
    test_data = np.array(["a", "b", "c", "x", "!"])
    encoded_data_control = encoder.encode(test_data)
    encoder.touch()
    encoder.save()
    assert os.path.exists(encoder.save_abspath)
    encoder_loaded = BaseExecutor.load(encoder.save_abspath)
    encoded_data_test = encoder_loaded.encode(test_data)
    np.testing.assert_array_equal(encoded_data_control, encoded_data_test)
    rm_files(
        [
            encoder.config_abspath,
            encoder.save_abspath,
            encoder_loaded.config_abspath,
            encoder_loaded.save_abspath,
        ]
    )


@pytest.mark.parametrize("encoder", encoders)
def test_save_and_load_config(encoder):
    encoder.save_config()
    assert os.path.exists(encoder.config_abspath)
    encoder_loaded = BaseExecutor.load_config(encoder.config_abspath)
    rm_files([encoder_loaded.config_abspath, encoder_loaded.save_abspath])



import io
import os

import numpy as np
from PIL import Image

from .. import ImageReader


def create_test_image(output_fn, size_width=50, size_height=50):
    from PIL import Image
    image = Image.new('RGB', size=(size_width, size_height), color=(155, 0, 0))
    with open(output_fn, "wb") as f:
        image.save(f, 'jpeg')


def test_io_uri():
    crafter = ImageReader()
    tmp_fn = os.path.join(crafter.current_workspace, 'test.jpeg')
    img_size = 50
    create_test_image(tmp_fn, size_width=img_size, size_height=img_size)
    test_doc = crafter.craft(buffer=None, uri=tmp_fn)
    assert test_doc['blob'].shape == (img_size, img_size, 3)


def test_io_buffer():
    crafter = ImageReader()
    tmp_fn = os.path.join(crafter.current_workspace, 'test.jpeg')
    img_size = 50
    create_test_image(tmp_fn, size_width=img_size, size_height=img_size)
    image_buffer = io.BytesIO()
    img = Image.open(tmp_fn)
    img.save(image_buffer, format='PNG')
    image_buffer.seek(0)
    test_doc = crafter.craft(buffer=image_buffer.getvalue(), uri=None)
    assert test_doc['blob'].shape == (img_size, img_size, 3)
    np.testing.assert_almost_equal(test_doc['blob'], np.array(img).astype('float32'))



__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
__license__ = "Apache-2.0"

# lift the chunk-level topk to doc-level topk
from typing import Iterable

import numpy as np

from . import BaseExecutableDriver
from .helper import pb_obj2dict

if False:
    from ..proto import jina_pb2


class BaseRankDriver(BaseExecutableDriver):
    """Drivers inherited from this Driver will bind :meth:`rank` by default """

    def __init__(self, executor: str = None, method: str = 'score', *args, **kwargs):
        super().__init__(executor, method, *args, **kwargs)
        self._is_apply = False


class Chunk2DocRankDriver(BaseRankDriver):
    """Extract matches score from chunks and use the executor to compute the rank and assign the resulting matches to the
    level above.

    Note that it traverses on ``chunks`` not on ``matches``

    Input-Output ::
        Input:
        document: {granularity: k-1}
                |- chunks: {granularity: k}
                |    |- matches: {granularity: k}
                |
                |- chunks: {granularity: k}
                    |- matches: {granularity: k}
        Output:
        document: {granularity: k-1}
            |- chunks: {granularity: k}
            |    |- matches: {granularity: k}
            |
            |- chunks: {granularity: k}
            |    |- matches: {granularity: k}
            |
            |-matches: {granularity: k-1} (Ranked according to Ranker Executor)
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, recur_on='chunks', **kwargs)
        self.recursion_order = 'post'

    def _apply_all(self, docs: Iterable['jina_pb2.Document'], context_doc: 'jina_pb2.Document', *args, **kwargs):
        """

        :param docs: the chunks of the ``context_doc``, they are at depth_level ``k``
        :param context_doc: the owner of ``docs``, it is at depth_level ``k-1``
        :return:
        """

        # if at the top-level already, no need to aggregate further
        if context_doc is None:
            return

        match_idx = []
        query_chunk_meta = {}
        match_chunk_meta = {}
        for c in docs:
            for match in c.matches:
                match_idx.append((match.parent_id, match.id, c.id, match.score.value))
                query_chunk_meta[c.id] = pb_obj2dict(c, self.exec.required_keys)
                match_chunk_meta[match.id] = pb_obj2dict(match, self.exec.required_keys)

        # np.uint32 uses 32 bits. np.float32 uses 23 bit mantissa, so integer greater than 2^23 will have their
        # least significant bits truncated.
        if match_idx:
            match_idx = np.array(match_idx, dtype=np.float64)

            docs_scores = self.exec_fn(match_idx, query_chunk_meta, match_chunk_meta)
            for doc_id, score in docs_scores:
                r = context_doc.matches.add()
                r.id = int(doc_id)
                r.granularity = context_doc.granularity
                r.adjacency = context_doc.adjacency + 1
                r.score.ref_id = context_doc.id  # label the score is computed against doc
                r.score.value = score
                r.score.op_name = exec.__class__.__name__


class CollectMatches2DocRankDriver(BaseRankDriver):
    """This Driver is intended to take a `document` with matches at a `given level depth > 0`, clear those matches and substitute
    these matches by the documents at a lower depth level.
    Input-Output ::
        Input:
        document: {granularity: k}
            |- matches: {granularity: k}
        Output:
        document: {granularity: k}
            |- matches: {granularity: k-1} (Sorted according to Ranker Executor)

    Imagine a case where we are querying a system with text documents chunked by sentences. When we query the system,
    we use sentences (chunks) to query it. So at some point we will have:
    `query sentence (documents of granularity 1):
        matches: indexed sentences (documents of level depth 1)`
    `
    But in the output we want to have the full document that better matches the `sentence`.
    `query sentence (documents of granularity 1):
        matches: indexed full documents (documents of level depth 0).
    `
    Using this Driver before querying a Binary Index with full binary document data can be very useful to implement a search system.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.recursion_order = 'post'

    def _apply_all(self, docs: Iterable['jina_pb2.Document'], context_doc: 'jina_pb2.Document', *args, **kwargs):
        """

        :param docs: the chunks of the ``context_doc``, they are at depth_level ``k``
        :param context_doc: the owner of ``docs``, it is at depth_level ``k-1``
        :return:
        """

        # if at the top-level already, no need to aggregate further
        if context_doc is None:
            return

        match_idx = []
        query_chunk_meta = {}
        match_chunk_meta = {}
        # doc_id_to_match_map = {}
        for match in context_doc.matches:
            # doc_id_to_match_map[match.id] = index
            match_idx.append((match.parent_id, match.id, context_doc.id, match.score.value))
            query_chunk_meta[context_doc.id] = pb_obj2dict(context_doc, self.exec.required_keys)
            match_chunk_meta[match.id] = pb_obj2dict(match, self.exec.required_keys)

        if match_idx:
            match_idx = np.array(match_idx, dtype=np.float64)

            docs_scores = self.exec_fn(match_idx, query_chunk_meta, match_chunk_meta)
            # These ranker will change the current matches
            context_doc.ClearField('matches')
            for doc_id, score in docs_scores:
                r = context_doc.matches.add()
                r.id = int(doc_id)
                r.score.ref_id = context_doc.id  # label the score is computed against doc
                r.score.value = score
                r.score.op_name = exec.__class__.__name__



import numpy as np
import os
import mock
import shutil
import pytest

from .. import TextPaddlehubEncoder
from jina.executors import BaseExecutor


target_output_dim = 1024
test_data = np.array(['it is a good day!', 'the dog sits on the floor.'])
tmp_files = []


def teardown():
    for k in tmp_files:
        if os.path.exists(k):
            if os.path.isfile(k):
                os.remove(k)
            elif os.path.isdir(k):
                shutil.rmtree(k, ignore_errors=False, onerror=None)


def add_tmpfile(*path):
    tmp_files.extend(path)


class MockModule:
    def get_embedding(self, texts, *args, **kwargs):
        print('i am a mocker embedding')
        return [[np.random.random(target_output_dim), None]] * len(texts)


def _test_textpaddlehubencoder_encode():
    encoder = TextPaddlehubEncoder()
    encoded_data = encoder.encode(test_data)
    assert encoded_data.shape == (2, target_output_dim)
    add_tmpfile(encoder.save_abspath, encoder.config_abspath)
    teardown()


@mock.patch('paddlehub.Module', return_value=MockModule())
def test_textpaddlehubencoder_encode(mocker):
    _test_textpaddlehubencoder_encode()


@mock.patch('paddlehub.Module', return_value=MockModule())
def test_textpaddlehubencoder_save_and_load(mocker):
    encoder = TextPaddlehubEncoder()
    encoder.touch()
    encoder.save()
    assert os.path.exists(encoder.save_abspath)
    encoder_loaded = BaseExecutor.load(encoder.save_abspath)
    assert encoder_loaded.model_name == encoder.model_name
    add_tmpfile(encoder.save_abspath, encoder.config_abspath)
    teardown()


@mock.patch('paddlehub.Module', return_value=MockModule())
def test_textpaddlehubencoder_save_and_load_config(mocker):
    encoder = TextPaddlehubEncoder()
    encoder.save_config()
    assert os.path.exists(encoder.config_abspath)
    encoder_loaded = BaseExecutor.load_config(encoder.config_abspath)
    assert encoder_loaded.model_name == encoder.model_name
    add_tmpfile(encoder.save_abspath, encoder.config_abspath)
    teardown()

@pytest.mark.skipif('JINA_TEST_PRETRAINED' not in os.environ, reason='skip the pretrained test if not set')
def test_textpaddlehubencoder_encode_with_pretrained_model():
    _test_textpaddlehubencoder_encode()



import os
import mock
import numpy as np
import shutil
import pytest

from .. import FarmTextEncoder
from jina.executors import BaseExecutor
from jina.executors.metas import get_default_metas

def get_metas():
    metas = get_default_metas()
    if 'JINA_TEST_GPU' in os.environ:
        metas['on_gpu'] = True
    return metas

def rm_files(tmp_files):
    for file in tmp_files:
        if file and os.path.exists(file):
            if os.path.isfile(file):
                os.remove(file)
            elif os.path.isdir(file):
                shutil.rmtree(file, ignore_errors=False, onerror=None)

class MockModule:
    def __init__(self):
        pass

    def extract_vectors(self, dicts, *args, **kwargs):
        print('i am a mocker function')
        for k in dicts:
            k['vec'] = np.array([i for i in range(target_output_dim)])
        return dicts

target_output_dim = 768
test_data = np.array(['it is a good day!', 'the dog sits on the floor.'])

def _test_encoding_results():
    metas = get_metas()
    encoder = FarmTextEncoder(metas=metas)
    encoded_data = encoder.encode(test_data)
    assert encoded_data.shape == (2, target_output_dim)



@pytest.mark.skipif('JINA_TEST_PRETRAINED' not in os.environ, reason='skip the pretrained test if not set')
def test_encoding_results():
    _test_encoding_results()

@mock.patch('farm.infer.Inferencer.load', return_value=MockModule())
def test_encoding_results_local(mocker):
    _test_encoding_results()

@mock.patch('farm.infer.Inferencer.load', return_value=MockModule())
def test_save_and_load(mocker):
    metas = get_metas()
    encoder = FarmTextEncoder(metas=metas)
    encoded_data_control = encoder.encode(test_data)
    encoder.touch()
    encoder.save()
    assert os.path.exists(encoder.save_abspath)
    encoder_loaded = BaseExecutor.load(encoder.save_abspath)
    encoded_data_test = encoder_loaded.encode(test_data)
    assert encoder_loaded.pretrained_model_name_or_path == encoder.pretrained_model_name_or_path
    np.testing.assert_array_equal(encoded_data_control, encoded_data_test)
    rm_files([encoder.save_abspath])

@mock.patch('farm.infer.Inferencer.load', return_value=MockModule())
def test_save_and_load_config(mocker):
    metas = get_metas()
    encoder = FarmTextEncoder(metas=metas)
    encoder.save_config()
    assert os.path.exists(encoder.config_abspath)
    encoder_loaded = BaseExecutor.load_config(encoder.config_abspath)
    assert encoder_loaded.pretrained_model_name_or_path == encoder.pretrained_model_name_or_path
    rm_files([encoder.config_abspath])
